#include  <stdio.h>
#include <string.h>
#include "snes_rom_header.h"

int parse_params(char *arg);
int process_filename(char *filename);
void print_header_list_mode(char *filename, struct snes_rom_header *header);
void print_header_normal_mode(char *filename, struct snes_rom_header *header);
static char *trim_path(char *path);

char list_mode=0; // default, normal mode
char html_mode=0; // default off

int main(int argc, char **argv)
{
	int i, num_files, start;

	if (argc<2) {
			printf("snesrom_info version %s, by Raphael Assenat\n\n", PROG_VERSION);
			printf("Usage: ./snesrom_info [options] filenames ...\n");
			printf("\nOptions:\n");
			printf(" l   display info in list mode(on same row)\n");
			printf(" h   html mode\n");
			printf("\n   eg: ./snesrom_info l donkey1.smc\n");
			return -1;
	}

	num_files=argc-1; // substract the program filename
	start=1;

	
	if (argv[1][0]=='-') {
			parse_params(argv[1]);
			start=2;
			num_files--;
	}
	
	if (html_mode) {
		printf("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\">\n");
		printf("<html>\n<head>\n");
		printf("<title>Rom List</title>\n");
		printf("<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n");
		printf("</head>\n<body>");
		printf("<center><h1>Rom list</h1></center><br>\n<br>\n");
		if (list_mode) {
			printf("<table border=\"1\">\n");	
			printf("<tr><td><b>Filename</b></td><td><b>Title</b></td><td><b>Country</b></td><td><b>License</b></td></tr>\n");
		}
	}
	
	for (i=0; i<num_files; i++)
	{
		//printf("%s\n", argv[start+i]);
		process_filename(argv[argc-num_files+i]);	
	}		

	if (html_mode) {
		if (list_mode) { printf("</table><br>\n"); }
		printf("<br>\n");
		printf("<font size=\"-1\">List generated using snesrom_info version %s.<br>\n", PROG_VERSION);
		printf(
		"Latest version available at <a href=\"http://www.raphnet.net/programmation/mixed/mixed.html\">"
		"http://www.raphnet.net/programmation/mixed/mixed.html</a></font><br>\n");		
		printf("</body>\n</html>\n");
	}	
		
	return 0;		
}

int parse_params(char *arg)
{
	int optlen, i;
	
	optlen = strlen(arg);
	for (i=0; i<optlen; i++) {
		switch (arg[i])
		{
			case 'l': list_mode=1; break;
			case 'h': html_mode=1; break;
			case '-': break;
			default: return -1;
		}
	}
	return 0;
}

int process_filename(char *filename)
{
	struct snes_rom_header header;
	FILE *rom_file;
	int res;
	
	/* open the file */ 	
	rom_file = fopen(filename, "rb");
	if (rom_file==NULL) {
			perror("fopen");
			return -2;
	}
	
	/* find and read the header */
	res = find_and_read_header_from_file(rom_file, &header);
	if (res<0) {
		fprintf(stderr, "Could not find rom header\n");
		return -1;
	}

	filename = trim_path(filename);
	if (list_mode) {
		print_header_list_mode(filename, &header);
	} else {
		print_header_normal_mode(filename, &header);
	}	

	fclose(rom_file);

		
	return 0;
}

void print_header_list_mode(char *filename, struct snes_rom_header *header)
{
	if (html_mode) {
		printf("<tr><td>%s</td><td>%s</td><td>%s</td><td>%s</td></tr>\n",
					filename, header->title,
					get_country_string(header->country),
					get_license_string(header->license));
	} else {
		printf("%-25s  %-25s  %-15s  %s\n", 
					filename, header->title,
					get_country_string(header->country),
					get_license_string(header->license));
	}
}

void print_header_normal_mode(char *filename, struct snes_rom_header *header)
{
	if (html_mode) {
		printf("<b>Filename:</b> %s<br>\n", filename);
		printf("<b>Title:</b> %s<br>\n", header->title);	
		printf("<b>Type:</b> %s<br>\n", get_rom_type_string(header->rom_type));
		printf("<b>Country:</b> %s<br>\n", get_country_string(header->country));
		printf("<b>License:</b> %s<br>\n", get_license_string(header->license));
		printf("<hr>\n");
		
	} else {
		printf("Filename: %s\n", filename);
		printf("Title: %s\n", header->title);	
		printf("Type: %s\n", get_rom_type_string(header->rom_type));
		printf("Country: %s\n", get_country_string(header->country));
		printf("License: %s\n", get_license_string(header->license));
		printf("------------------\n");
	}
}

static char *trim_path(char *path)
{
	int len = strlen(path);
	int i;

	for (i=len-1; i>=0; i--)
	{
		if (path[i]=='/') { return &path[i+1]; }
	}	
	return path;
}

